/*
  lp_BLE_TempHumidity
  Reads RH/temp, saves historical values, displays value when connected to by pfodApp, www.pfod.com.au
  (c)2018 Forward Computing and Control Pty. Ltd.
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/

/** Supply figures (adjusted for MAX8881 5uA addition to test figures)
    for 2sec advertising interval ~17uA while not connected
    for min/max connections 100mS/150mS and slaveLatency 4 and 30sec display refresh ~25uA
    for each display of the hr or day plot uses ~0.001mAHr
    So if continually connected supply consumption is 0.72mAHr per day (and about 1/3 less if not connected)
    Coin Cell CR2032 235mAHr => 326days ~1year
    Coin Cell CR2450 650mAHr => 900days ~2.5years
    2 x  AAA Alkaline ~1200mAHr to 0.8V say less 20% for 1V cutoff => 1000mAHr => 1389days ~3.8years
    2 x AAA Lithium ~1300mAHr to 1V cutoff => 1805 days ~4.9years
    With Solar assist assume 8hr average per day solar power => 50% increase in battery life

 **/
#include "lp_Si7021.h"
#include <Wire.h>
#include "Button.h"
#include "RHGauge.h"
#include "Thermometer.h"

char bleNameStr[] = "Car Temp/RH"; // <<<< set BLE advert name here
uint16_t humidityRaw = 0;
uint16_t tempCRaw = 0;
bool sensorIntialized = false; // if not true do not read values

// store history every 40min for 10days => 360 points
// when connected measure every 30sec to update screen or when screen above buttons is clicked or when the main screen is re-displayed

//#define DEBUG
//#define DUMMY_READINGS

// This sketch needs the Low Power nRF52 installed and compile using Arduino V1.8.2 IDE
// see https://www.forward.com.au/pfod/BLE/LowPower/index.html
#include <lp_BLESerial.h>

// download the pfodParser library V3.10+ from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
// pfodParse.h  includes WStream and then defines Stream as WStream to match the Redbear BLE hardware files and to avoid conflict with mbed::Stream
#include <pfodParser.h>
#include <pfodDwgs.h>

// download the libraries from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
// pfodParser.zip V3.31+ contains pfodParser, pfodSecurity, pfodDelay, pfodBLEBufferedSerial, pfodSMS and pfodRadio

pfodParser parser("V1"); // create a parser to handle the pfod messages
pfodDwgs dwgs(&parser);  // drawing support
lp_BLESerial bleSerial; // create a BLE serial connection

Thermometer thermometer = Thermometer(&dwgs);
RHGauge rhGauge = RHGauge(&dwgs);
Button hrs8PlotButton = Button('m', &dwgs);
Button days1PlotButton =  Button('h', &dwgs);
Button days3PlotButton = Button('d', &dwgs);
Button days10PlotButton =  Button('w', &dwgs);

// there for 1 reading consumes
lp_Si7021 sensor;
lp_timer sensorTimer;
const unsigned long SAMPLE_DELAY_TIME = 90 * 1000; // mS == 1.5mins
const uint16_t ARRAY_SIZE = 120;// total size 2*5*120*2bytes = 2.3K
const uint16_t FINE_PRINT_SIZE = 30;

// allocate arrays for raw readings uint16_t

struct struct_readings {
  float deltaT_mins; // time step in mins
  float printTime; // for print idx
  float printFactor; // 1 for mins, 60 for hrs, 60*24 for days
  uint16_t arraySize; // size of temp and RH arrays
  uint16_t *Temp;
  uint16_t *RH;
  bool printReadingsFinished;// = false; // these are not actually used as first plot is 9hrs at 3min intervals with 1min for first 135 readings
  bool printInitialized;// = false;
  bool printingFine; // true when showing first 135 point from fine readings
  uint16_t nextIdx;// = 0; // current index for storing next reading
  uint16_t printCount;// = 0;
  uint16_t printSize; // = 0;
  uint16_t printIdx;// = 0;
  uint16_t startIdx;// = 0;
  uint16_t printFineIdx;// = 0;
  uint16_t startFineIdx;// = 0;
  const char* dataPadding; // string to add to plot data to distingush the 4 plots
  const char* name; // name of this readings
};

struct struct_readings readings_1min;
const char dataPadding_1min[] = ""; // nothing not used
const char name_1min[] = "3hr";
uint16_t Temp_1min[ARRAY_SIZE];
uint16_t RH_1min[ARRAY_SIZE];

struct struct_readings readings_3min;
const char dataPadding_3min[] = "";
const char name_3min[] = "_8hrs";
uint16_t Temp_3min[ARRAY_SIZE];
uint16_t RH_3min[ARRAY_SIZE];

struct struct_readings readings_9min;
const char dataPadding_9min[] = ",m";
const char name_9min[] = "__24hrs";
uint16_t Temp_9min[ARRAY_SIZE];
uint16_t RH_9min[ARRAY_SIZE];

struct struct_readings readings_27min;
const char dataPadding_27min[] = ",,h";
const char name_27min[] = "___3day";
const uint16_t arraySize_3day = 288;
uint16_t Temp_27min[arraySize_3day];
uint16_t RH_27min[arraySize_3day];

struct struct_readings readings_81min;
const char dataPadding_81min[] = ",,,w";
const char name_81min[] = "____8day";


const uint16_t SIZE_OF_STRUCTS_ARRAY = 4;
struct struct_readings* arrayOfStructReadings[SIZE_OF_STRUCTS_ARRAY];

float Time_minPerHr = 1.0f / 60.0f;
float Time_minPerDay = Time_minPerHr / 24.0f;

lp_timer readResultDelay;
const unsigned long READ_RESULT_DELAY_TIME = 25; // mS

lp_timer PrintDelay;
const unsigned long PRINT_DELAY_TIME = 200; // mS

bool sendReadingsPaused = false;

void incrementNextIdx(struct_readings* readingsPtr) {
  uint16_t idx = readingsPtr->nextIdx;
  idx++;
  if (idx >= readingsPtr->arraySize) {
    idx = 0; // loop
  }
  readingsPtr->nextIdx = idx;
}

uint16_t decrementIdx(struct_readings* readingsPtr, uint16_t idx) {
  if (idx == 0) {
    idx = readingsPtr->arraySize; // loop
  }
  idx--;
  return idx;
}

void updateReadings(uint16_t tempCRaw, uint16_t humidityRaw) {
#ifdef DUMMY_READINGS
  tempCRaw += 10; // make sure any sensor reading is not zero
#endif
  struct_readings* readingsPtr;
  uint16_t arrayIdx = 0;
  uint16_t currentIdx; // index of last stored value
  for (; arrayIdx < SIZE_OF_STRUCTS_ARRAY; arrayIdx++) {
    readingsPtr = arrayOfStructReadings[arrayIdx];
    currentIdx = readingsPtr->nextIdx;
    readingsPtr->Temp[currentIdx] = tempCRaw;
    readingsPtr->RH[currentIdx] = humidityRaw;
    // if get new reading while sending history, update startHrIdx to prevent new reading being shown at end of history
    readingsPtr->startIdx = currentIdx;
    if (readingsPtr->printReadingsFinished || sendReadingsPaused) {
      // user has already displayed a plot
      // reset plotting to show latest readings
      stopSending(readingsPtr);
    }

    incrementNextIdx(readingsPtr); // move on to next idx
    if ((currentIdx % 3) != 0) { // is last stored was multiple of 3 loop to add to next set of readings
      break; // finished
    }
    // else loop and add to next set of readings
  }
  // always keep these in sync
  if (readingsPtr == &readings_27min) {
    // just updated readings_27min
    readings_81min.nextIdx = readings_27min.nextIdx;
    readings_81min.startIdx = readings_27min.startIdx;
    if (readings_81min.printReadingsFinished || sendReadingsPaused) {
      // user has already displayed a plot
      // reset plotting to show latest readings
      stopSending(&readings_81min);
    }
  }
}

void initializeArray(struct_readings* readings) {
  for (uint16_t i = 0; i < ARRAY_SIZE; i++) {
    readings->Temp[i] = readings->RH[i] = 0; // 0 is invalid
  }
  readings->printFactor = 1.0f; //default
  readings->nextIdx = 0; // current index for storing next reading
  readings->printReadingsFinished = false;
  readings->printInitialized = false;
}

#ifdef DUMMY_READINGS
void dummyReadings() {
  uint16_t base = 1000;
  int noReadings = 44 * 180 + 3;
  for (int i = 0; i < noReadings; i++) {
    updateReadings(base, base);
    base += 10;
  }
}
#endif

void setup() {
#ifdef DEBUG
  Serial.begin(115200);
  Serial.println();
  for (int i = 10; i > 0; i--) {
    Serial.print(' '); Serial.print(i);
    delay(500);
  }
  Serial.println();
#endif
  //Initialize the sensor and ping it
  if (sensor.begin()) {
#ifdef DEBUG
    Serial.println("Found Si7021");
#endif
    sensorIntialized = true; // else leave as false
  } else {
#ifdef DEBUG
    Serial.println("Error: Did not find Si7021");
#endif
  }
  rhGauge.setLabel(F("RH\n"));
  rhGauge.setValue(0);
  thermometer.setValue(0); // 150 == 27.5. -30 = -17.5, 300 = 60.5
  hrs8PlotButton.setLabel(F("Last 8 hrs"));
  days1PlotButton.setLabel(F("Last 24 hrs"));
  days3PlotButton.setLabel(F("Last 3 days"));
  days10PlotButton.setLabel(F("Last 7 days"));


  initializeArray(&readings_1min);
  readings_1min.dataPadding = dataPadding_1min;
  readings_1min.name = name_1min;
  readings_1min.deltaT_mins = SAMPLE_DELAY_TIME / (60.0 * 1000.0);
  readings_1min.printFactor = 1.0f; // mins
  readings_1min.Temp = Temp_1min;
  readings_1min.RH = RH_1min;
  readings_1min.arraySize = ARRAY_SIZE;
  readings_1min.printSize = ARRAY_SIZE;

  initializeArray(&readings_3min);
  readings_3min.dataPadding = dataPadding_3min;
  readings_3min.name = name_3min;
  readings_3min.deltaT_mins = 3 * readings_1min.deltaT_mins;
  readings_3min.printFactor = 60.0f; // hrs
  readings_3min.Temp = Temp_3min;
  readings_3min.RH = RH_3min;
  readings_3min.arraySize = ARRAY_SIZE;
  readings_3min.printSize = ARRAY_SIZE;

  initializeArray(&readings_9min);
  readings_9min.dataPadding = dataPadding_9min;
  readings_9min.name = name_9min;
  readings_9min.deltaT_mins = 3 * readings_3min.deltaT_mins;
  readings_9min.printFactor = 60.0f; // hrs
  readings_9min.Temp = Temp_9min;
  readings_9min.RH = RH_9min;
  readings_9min.arraySize = ARRAY_SIZE;
  readings_9min.printSize = ARRAY_SIZE;

  initializeArray(&readings_27min);
  readings_27min.dataPadding = dataPadding_27min;
  readings_27min.name = name_27min;
  readings_27min.deltaT_mins = 3 * readings_9min.deltaT_mins;
  readings_27min.printFactor = 60.0f * 24.0f; // days
  readings_27min.Temp = Temp_27min;
  readings_27min.RH = RH_27min;
  readings_27min.arraySize = arraySize_3day;
  readings_27min.printSize = ARRAY_SIZE; // limit this print to 120

  initializeArray(&readings_81min);
  readings_81min.dataPadding = dataPadding_81min;
  readings_81min.name = name_81min;
  readings_81min.deltaT_mins = readings_27min.deltaT_mins;
  readings_81min.printFactor = 60.0f * 24.0f; // days
  readings_81min.Temp = Temp_27min;
  readings_81min.RH = RH_27min;
  readings_81min.arraySize = arraySize_3day;
  readings_81min.printSize = arraySize_3day;

  arrayOfStructReadings[0] = &readings_1min;
  arrayOfStructReadings[1] = &readings_3min;
  arrayOfStructReadings[2] = &readings_9min;
  arrayOfStructReadings[3] = &readings_27min;
  // arrayOfStructReadings[4] = &readings_81min;


  // set advertised name
  bleSerial.setName(bleNameStr);  // <<<<<<<< set your device name here
  // begin BLE initialization
  // BLE advertising and connection defaults are in bleConstants.h

  bleSerial.setAdvertisingInterval(2000); // in mS, default is 500, 1000 gives ~34uA while advertising, 2000 gives ~22uA  while advertising
  // 500 to 1000 is the suggested value for advertising interval

  // connection Interval
  // default is 100mS min 150mS max
  // bleSerial.setConnectionInterval(100,150); //leave as default, this suggests a connection interval to be used by the Android mobile,
  // the Android mobile actually chooses the connection interval to be used
  // while downloading 36hrs plot have ~110uA for 45sec => extra 85uA for 45sec == ~0.001mAHr per plot display

  // set the slave latency default is 0, i.e. respond to every connection msg
  bleSerial.setSlaveLatency(4); //default is 0, 0=>~50uA, 2=>~30uA, 4 => ~25uA + refresh every 30sec 66uA for 2sec => ~4.5uA each 30sec refresh for total of ~30uA connected

  bleSerial.begin();

  // you can set TX power here AFTER calling bleSerial.begin()
  // default is +4
  // bleSerial.setTxPower(4);

  parser.connect(&bleSerial);

  if (sensorIntialized) {
    sensorTimer.startTimer(SAMPLE_DELAY_TIME, startRHandTempConversion);
  }
  // do an initial sensor reading to initialize WIRE interface, otherwise will draw 218uA until first reading
  startRHandTempConversion();

#ifdef DUMMY_READINGS
  dummyReadings();
#endif

#ifdef DEBUG
  Serial.println("setup() finished");
#endif
}

bool dwgRefresh = false;

struct_readings* currentReadings; // stores arrays
struct_readings* currentReadingsFine; // stores arrays
struct_readings* currentReadingsPrint; // stores indices

void printHistoricalReadings(struct_readings * readingsPrint, struct_readings * readings, struct_readings * readingsFine) {
#ifdef DEBUG
  Serial.print("printHistoricalReadings "); Serial.print(" Print:"); Serial.print(readingsPrint->name); Serial.print(" Readings:"); Serial.print(readings->name); Serial.print(" Fine:"); Serial.print(readingsFine->name);  Serial.println();
#endif
  // first 30 readings come from readingsFine, then start at 10 (zero based) offset into reading for next 110 readings
  // print then 1 line per 0.2 sec
  // start at previous nextIdx and go back in time
  readingsPrint->printCount = 0;
  readingsPrint->printTime = 0.0;
  readingsPrint->printIdx = decrementIdx(readings, readingsPrint->nextIdx); // most current reading
  readingsPrint->startIdx = readingsPrint->printIdx; // stop when we get here again
  readingsPrint->printFineIdx = decrementIdx(readingsFine, readingsFine->nextIdx); // most current reading
#ifdef DEBUG
  Serial.print(readingsFine->name); Serial.print(" initial readingsPrint->printFineIdx:"); Serial.print(readingsPrint->printFineIdx); Serial.println();
#endif
  while ((readingsPrint->printFineIdx) % 3 != 0) {
    // start with 1 so that after 30 prints will be aligned with next higher set of readings
    readingsPrint->printFineIdx = decrementIdx(readingsFine, readingsPrint->printFineIdx); // most current reading
  }
#ifdef DEBUG
  Serial.print(readingsFine->name); Serial.print(" after reduction readingsPrint->printFineIdx:"); Serial.print(readingsPrint->printFineIdx); Serial.println();
#endif
  readingsPrint->startFineIdx = readingsPrint->printFineIdx; // stop when we get here again
  readingsPrint->printInitialized = true;
  readingsPrint->printingFine = true;
  sendReadingsPaused = false;
  // set globals
  currentReadings = readings;
  currentReadingsFine = readingsFine;
  currentReadingsPrint = readingsPrint;
#ifdef DEBUG
  Serial.print(readings->name); Serial.print(" startIdx:");  Serial.print(readings->startIdx);  Serial.print(" printSize:"); Serial.print(currentReadingsPrint->printSize); Serial.println();
#endif
  printNextReading();
}

void printNextReading() {
#ifdef DEBUG
  Serial.print("printNextReading For  "); Serial.print(currentReadingsPrint->name); Serial.println();
#endif
  if (currentReadingsPrint->printReadingsFinished) {
    return;
  }
  if (sendReadingsPaused) {
    return;
  }
  bool sentReading = false;

  if (currentReadingsPrint->printingFine) {
#ifdef DEBUG
    Serial.print("printing Fine "); Serial.print(currentReadingsFine->name); Serial.print(" Temp:"); Serial.print(currentReadingsFine->Temp[currentReadingsPrint->printFineIdx]); Serial.println();
#endif
    if (currentReadingsFine->Temp[currentReadingsPrint->printFineIdx] != 0) { // skip invalid reading in first hr
      parser.print(currentReadingsPrint->printTime / currentReadingsPrint->printFactor, 3); // time in mins/hrs/days as set by readings 3decimals
      parser.print(','); parser.print(sensor.Temp_RawToFloat(currentReadingsFine->Temp[currentReadingsPrint->printFineIdx]));
      parser.print(','); parser.print(sensor.RH_RawToInt(currentReadingsFine->RH[currentReadingsPrint->printFineIdx]));
      parser.print(currentReadingsPrint->dataPadding); // same padding for fine
      parser.println(); // end of CSV data record
      sentReading = true;
#ifdef DEBUG
      Serial.print(" sendReading:"); Serial.print(sentReading);
      Serial.print(" printFineIdx:"); Serial.print(currentReadingsPrint->printFineIdx);
      Serial.print(" Time:"); Serial.print(currentReadingsPrint->printTime / currentReadingsPrint->printFactor, 3);;
      Serial.print(" Temp:"); Serial.print(currentReadingsFine->Temp[currentReadingsPrint->printFineIdx]);
      Serial.println();
#endif
      currentReadingsPrint->printFineIdx = decrementIdx(currentReadingsFine, currentReadingsPrint->printFineIdx);
      currentReadingsPrint->printCount++;
      if (currentReadingsPrint->printCount == FINE_PRINT_SIZE + 1) {
        currentReadingsPrint->printingFine = false; // finished
        currentReadingsPrint->printCount = FINE_PRINT_SIZE / 3;
        // skip the first currentReadings->printCount readings
#ifdef DEBUG
        Serial.print("printCount "); Serial.print(currentReadings->printCount); Serial.println();
#endif
        for (int i = currentReadingsPrint->printCount; i >= 0; i--) {
          currentReadingsPrint->printIdx = decrementIdx(currentReadings, currentReadingsPrint->printIdx);
        }
        //currentReadingsPrint->printIdx = decrementIdx(currentReadings, currentReadingsPrint->printIdx);
        currentReadingsPrint->printTime -= currentReadingsPrint->deltaT_mins;
#ifdef DEBUG
        Serial.print(" Next printIdx:"); Serial.print(currentReadingsPrint->printIdx);
        Serial.print(" Time:"); Serial.print(currentReadingsPrint->printTime / currentReadingsPrint->printFactor, 3);;
        Serial.print(" Temp:"); Serial.print(currentReadings->Temp[currentReadingsPrint->printIdx]);
        Serial.println();
#endif
      } else {
        currentReadingsPrint->printTime -= currentReadingsFine->deltaT_mins;
      }

    }
  } else { // not printing fine
#ifdef DEBUG
    Serial.print("printing "); Serial.print(currentReadings->name); Serial.println();
#endif
    if (currentReadingsPrint->Temp[currentReadingsPrint->printIdx] != 0) { // skip invalid reading in first hr
      parser.print(currentReadingsPrint->printTime / currentReadingsPrint->printFactor, 3); // time in mins
      parser.print(','); parser.print(sensor.Temp_RawToFloat(currentReadings->Temp[currentReadingsPrint->printIdx]));
      parser.print(','); parser.print(sensor.RH_RawToInt(currentReadings->RH[currentReadingsPrint->printIdx]));
      parser.print(currentReadingsPrint->dataPadding); // same padding for fine
      parser.println(); // end of CSV data record
      sentReading = true;
#ifdef DEBUG
      Serial.print(" sendReading:"); Serial.print(sentReading);
      Serial.print(" printIdx:"); Serial.print(currentReadingsPrint->printIdx);
      Serial.print(" Time:"); Serial.print(currentReadingsPrint->printTime / currentReadingsPrint->printFactor, 3);;
      Serial.print(" Temp:"); Serial.print(currentReadings->Temp[currentReadingsPrint->printIdx]);
      Serial.println();
#endif

      currentReadingsPrint->printIdx = decrementIdx(currentReadings, currentReadingsPrint->printIdx);
      currentReadingsPrint->printCount++;
      currentReadingsPrint->printTime -= currentReadings->deltaT_mins;
    }
  }
#ifdef DEBUG
  Serial.print("sentReading "); Serial.print(sentReading); Serial.print(" "); Serial.print(currentReadingsPrint->printIdx); Serial.print(" != "); Serial.print(currentReadingsPrint->startIdx); Serial.println();
#endif
  if (sentReading && (currentReadingsPrint->printCount < currentReadingsPrint->printSize) && ((currentReadingsPrint->printIdx != currentReadingsPrint->startIdx) || (currentReadingsPrint->printingFine))  ) {
#ifdef DEBUG
    Serial.print("startDelay "); Serial.println();
#endif
    PrintDelay.startDelay(PRINT_DELAY_TIME, printNextReading);
  } else {
    // finished
#ifdef DEBUG
    Serial.print("print finished "); Serial.println();
#endif
    currentReadingsPrint->printReadingsFinished = true;
  }

}

void loop() {
  sleep(); // just sleep here waiting for the timer to trigger
  // check ble serial when triggered by anything
  uint8_t cmd = parser.parse(); // parse incoming data from connection
  // parser returns non-zero when a pfod command is fully parsed
  if (cmd != 0) { // have parsed a complete msg { to }
    uint8_t* pfodFirstArg = parser.getFirstArg(); // may point to \0 if no arguments in this msg.
    pfod_MAYBE_UNUSED(pfodFirstArg); // may not be used, just suppress warning
    long pfodLongRtn; // used for parsing long return arguments, if any
    pfod_MAYBE_UNUSED(pfodLongRtn); // may not be used, just suppress warning
    if ('.' == cmd) {
      // pfodApp has connected and sent {.} , it is asking for the main menu
      pauseSendingReadings();
      if (!parser.isRefresh()) {
        sendMainMenu(); // send back the menu designed
      } else {
        sendMainMenuUpdate(); // menu is cached just send update
      }

    } else if ('A' == cmd) {
      // in the main Menu of Menu_1
      byte dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (dwgCmd == hrs8PlotButton.getCmd()) { // 8hrs plot at 3min intervals + 1mins
        if (!(readings_3min.printInitialized)) {
          // stop all plots
          //stopSendingReadings(); // fix this later
          // clear plot and start again with latest data
          parser.print(F("{=8hrs Temperature / RH~C|time (hrs)~0~-0.5|Temp C~32~8~deg C|RH %~87~37~%}"));
          printHistoricalReadings(&readings_3min, &readings_3min, &readings_1min ); // start printing readings
        } else {
          // continue with existing data
          parser.print(F("{=8hrs Temperature / RH|time (hrs)~0~-0.5|Temp C~32~8~deg C|RH %~87~37~%}"));
          sendReadingsPaused = false;
          currentReadings = &readings_3min;
          currentReadingsFine = &readings_1min;
          currentReadingsPrint = &readings_3min;
          printNextReading();
        }
      } else if (dwgCmd == days1PlotButton.getCmd()) { // 24hrs plot at 9min + 3mins
        if (!(readings_9min.printInitialized)) {
          // stop all plots
          //stopSendingReadings(); // fix this later
          // clear plot and start again with latest data
          parser.print(F("{=24hrs Temperature / RH~C|time (days)~0~-0.25|Temp C~32~8~deg C|RH %~87~37~%|}"));
          printHistoricalReadings(&readings_9min, &readings_9min, &readings_3min); // start printing readings
        } else {
          // continue with existing data
          parser.print(F("{=24hrs days Temperature / RH|time (days)~0~-0.25|Temp C~32~8~deg C|RH %~87~37~%|}"));
          sendReadingsPaused = false;
          currentReadings = &readings_9min;
          currentReadingsFine = &readings_3min;
          currentReadingsPrint = &readings_9min;

          printNextReading();
        }
      } else if (dwgCmd == days3PlotButton.getCmd()) { // 3day plot at 27min + 9mins
        if (!(readings_27min.printInitialized)) {
          // stop all plots
          //stopSending(&readings_27min); // fix this later
          // clear plot and start again with latest data
          parser.print(F("{=3 days Temperature / RH~C|time (days)~0~-0.25|Temp C~32~8~deg C|RH %~87~37~%||}"));
          printHistoricalReadings(&readings_27min, &readings_27min, &readings_9min); // start printing readings use smaller print size for this 3 day plot
        } else {
          // continue with existing data
          parser.print(F("{=3 days Temperature / RH|time (days)~0~-0.25|Temp C~32~8~deg C|RH %~87~37~%||}"));
          sendReadingsPaused = false;
          currentReadings = &readings_27min;
          currentReadingsFine = &readings_9min;
          currentReadingsPrint = &readings_27min;

          printNextReading();
        }
      } else if (dwgCmd == days10PlotButton.getCmd()) { // 10day plot at 81min + 27mins
        if (!(readings_81min.printInitialized)) {
          // stop all plots
          //stopSending(&readings_27min); // fix this later
          // clear plot and start again with latest data
          parser.print(F("{=7 days Temperature / RH~C|time (days)~0~-0.25|Temp C~32~8~deg C|RH %~87~37~%|||}"));
          printHistoricalReadings(&readings_81min, &readings_27min, &readings_9min); // start printing readings

        } else {
          // continue with existing data
          parser.print(F("{=7 days Temperature / RH|time (days)~0~-0.25|Temp C~32~8~deg C|RH %~87~37~%|||}"));
          sendReadingsPaused = false;
          currentReadings = &readings_27min;
          currentReadingsFine = &readings_9min;
          currentReadingsPrint = &readings_81min;

          printNextReading();
        }
      } else {
        dwgRefresh = true; // just send update
        startRHandTempDwgUpdateConversion(); // will send dwg update when conversion complete in 25mS
        //sendDrawingUpdates_z(); // update the drawing
      }
      // always send back a response or pfodApp will timeout

    } else if ('z' == cmd) {
      pauseSendingReadings();
      // get starting offset
      dwgRefresh = parser.isRefresh(); // cleared when update sent
      startRHandTempDwgUpdateConversion(); // will send dwg update when conversion complete in 25mS

    } else if ('!' == cmd) {
      pauseSendingReadings();
      // CloseConnection command
      closeConnection(parser.getPfodAppStream());
    } else {
      // keep alive processed here so do not pauseSendingReadings();
      // or unknown command
      parser.print(F("{}")); // always send back a pfod msg otherwise pfodApp will disconnect.
    }
  }
  if (!bleSerial.isConnected()) {
    stopSendingReadings(); // reset -- next connection will send fill history
  }
}

void pauseSendingReadings() {
  sendReadingsPaused = true;
}

void stopSending(struct_readings * readings) {
  readings->printReadingsFinished = false; // stop sending readings
  readings->printInitialized = false; // re-initialize next connection
}

void stopSendingReadings() {
  // stops sending all readings
  stopSending(&readings_1min);
  stopSending(&readings_3min);
  stopSending(&readings_9min);
  stopSending(&readings_27min);
  stopSending(&readings_81min);
}


void closeConnection(Stream * io) {
  // add any special code here to force connection to be dropped
  stopSendingReadings();
  ((lp_BLESerial*)io)->close();
}

bool conversionRunning = false;
void startRHandTempConversion() {
  if (!conversionRunning) {
    conversionRunning = true;
    sensor.startRHandTemp();
  } // else just schedule to pickup last result
  readResultDelay.startDelay(READ_RESULT_DELAY_TIME, readRHResults);
}

void startRHandTempDwgUpdateConversion() {
  if (!conversionRunning) {
    conversionRunning = true;
    sensor.startRHandTemp();
  } // else just schedule to pickup last result
  readResultDelay.startDelay(READ_RESULT_DELAY_TIME, readRHResultsAndUpdateDwg);
}



uint16_t currentTemp_Raw;
uint16_t currentRH_Raw;

void readRHResultsAndUpdateDwg() {
  if (sensorIntialized) {
    // Get Result of Relative Humidity from or Si7021
    currentRH_Raw = sensor.readRH_Raw();
    // Read Result of Temperature from RH measurement Si7021
    currentTemp_Raw = sensor.readLastRHTemp_Raw();
  }
  conversionRunning = false;
  float temp = sensor.Temp_RawToFloat(currentTemp_Raw);
  int rh = sensor.RH_RawToInt(currentRH_Raw);
  // convert rh to in in range 0 to rhGauge.RANGE
  int rhValue = (int)(rh * (rhGauge.RANGE / 100.0f));
  rhGauge.setValue(rhValue);

  // convert temp into range 0 to Thermometer.RANGE
  int tempValue = floor(( (temp - thermometer.MIN_C) * (thermometer.RANGE / (thermometer.MAX_C - thermometer.MIN_C)) ) + 0.5);
  thermometer.setValue(tempValue);

#ifdef DEBUG
  Serial.print(" RH%: "); Serial.print(rh);
  Serial.print(" rhValue: "); Serial.print(rhValue);
  Serial.print(" degC: "); Serial.print(temp);
  Serial.print(" tempValue: "); Serial.print(tempValue);
  Serial.println();
  Serial.print("Temp:");  Serial.print(sensor.Temp_RawToFloat(currentTemp_Raw));  Serial.print("C, ");
  Serial.print("Humidity:");  Serial.print(sensor.RH_RawToInt(currentRH_Raw));  Serial.print("%");
  Serial.println();
#endif

  if (!dwgRefresh) {
    sendDrawing_z();
  } else {
    sendDrawingUpdates_z();
  }
  dwgRefresh = false;
}


void readRHResults() {
  if (sensorIntialized) {
    // Get Result of Relative Humidity from or Si7021
    humidityRaw = sensor.readRH_Raw();
    // Read Result of Temperature from RH measurement Si7021
    tempCRaw = sensor.readLastRHTemp_Raw();
  }
  conversionRunning = false;
  // Temperature is measured every time RH is requested.
  // It is faster, therefore, to read it from previous RH
  // measurement with readLastRHTemp() instead with startTemp() / readTemp()
  // save 1min reading and every 3 readings save in next array
  // i.e. save and increment and if new idx % 3 == 0 save in next array etc
#ifndef DUMMY_READINGS
  updateReadings(tempCRaw, humidityRaw);
#endif
}


void sendMainMenu() {
  // !! Remember to change the parser version string
  //    every time you edit this method
  parser.print(F("{,"));  // start a Menu screen pfod message
  // send menu background, format, prompt, refresh and version
  if (sensorIntialized) {
    parser.print(F("<bg w>~<+2><r><b>Temperature</r>\n <-2>and</-2> <bl><b>Rel Humidity")); // keep alive 5sec by default
  } else {
    parser.print(F("<bg w>~<+2><r><b>Error: Si7021\ndid not Initialize"));
  }
  parser.sendRefreshAndVersion(0); //  no refresh, dwg refreshes itself
  // send menu items
  parser.print(F("|+A~z")); //the dwg
  parser.print(F("}"));  // close pfod message
}

void sendMainMenuUpdate() {
  parser.print(F("{;"));  // start an Update Menu pfod message
  if (sensorIntialized) {
  } else {
    parser.print(F("<bg w>~<+2><r><b>Error: Si7021\ndid not Initialize"));
  }
  parser.print(F("|A")); //the dwg
  parser.print(F("}"));  // close pfod message
  // ============ end of menu ===========
}

void sendDrawing_z() {
  dwgs.start(50, 60, dwgs.WHITE); // background defaults to WHITE if omitted i.e.  start(50,35);
  parser.sendRefreshAndVersion(30000); // re-request dwg every 30sec sec. this is ignored if no parser version set
  // touch above buttons to force display updates
  dwgs.touchZone().cmd('u').size(50, 39).send();
  dwgs.pushZero(35, 22, 1.5); // move zero to centre of dwg to 35,22 and scale by 1.5 times
  rhGauge.draw(); // draw the control
  dwgs.popZero();
  dwgs.pushZero(18, 33); // move zero to centre of dwg to 18,33 scale is 1 (default)
  thermometer.draw(); // draw the control
  dwgs.popZero();

  dwgs.pushZero(12.5, 43, 0.7); // move zero to centre of dwg to 10,43 and scale by 0.8
  hrs8PlotButton.draw(); // draw the control
  dwgs.popZero();
  dwgs.pushZero(37.5, 43, 0.7); // move zero to centre of dwg to 25,43 and scale by 0.8
  days1PlotButton.draw(); // draw the control
  dwgs.popZero();

  dwgs.pushZero(12.5, 54, 0.7); // move zero to centre of dwg to 25,54 and scale by 0.8
  days3PlotButton.draw(); // draw the control
  dwgs.popZero();
  dwgs.pushZero(37.5, 54, 0.7); // move zero to centre of dwg to 25,54 and scale by 0.8
  days10PlotButton.draw(); // draw the control
  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_z() {
  dwgs.startUpdate();
  rhGauge.update(); // update with current state
  thermometer.update(); // update with current state
  dwgs.end();
}
